/*
 * Go implementation of Python script "largest":
 *   print 10 largest files in a tree (size and name)
 *
 *   R.Wobst, @(#) Dec 12 2011, 23:25:05
 */

package main

import (
    "os"
    "sort"
    "fmt"
    pf "path/filepath"
)

const N = 10

type Pair struct {size int64; filename string}
type Callback struct {pairlst []Pair; count int}

// the Sort interface

func (cb *Callback) Len() int {
    return N+1
}

func (cb *Callback) Less(i, j int) bool {
    return cb.pairlst[i].size > cb.pairlst[j].size	// invers sort!
}

func (cb *Callback) Swap(i, j int) {
    cb.pairlst[i], cb.pairlst[j] = cb.pairlst[j], cb.pairlst[i]
}

// the Visitor interface

func (cb *Callback) VisitDir(path string, f *os.FileInfo) bool {
    return true
}

func (cb *Callback) VisitFile(path string, f *os.FileInfo) {
    cb.pairlst[N].size, cb.pairlst[N].filename = f.Size, path
    cb.count++;
    sort.Sort(cb)
}

// ---------------------------------------------------------

func main() {
    cb := Callback{make([]Pair, N+1), 0}	// all zero and blank

    var err chan os.Error

    pf.Walk(".", &cb, err)

    fmt.Printf("%d files - 10 largest:\n\n", cb.count)

    for _, pair := range cb.pairlst[:10] {
	fmt.Printf("%8d bytes: %s\n", pair.size, pair.filename)
    }
}
